/**
 * Class, containing all actions of a player 
 *
 * @author Andrea Barth, Annika Niehage, Olga Mirkina
 * @version $Id: Action.java,v 1.3 2002/07/11 23:26:27 Niehage Exp $
 * 
 */


public class Action
{
/**
 * reference to the CommunicationChannel
 */
    private CommunicationChannel com;
    private String[] sendQueue = new String[3];
    private int sendStart = 0, sendEnd = 0;
    private boolean sendEmpty = true;
    private boolean sendAllowed = true;

    /**
     * creates a new instance of the action class
     * @param com is the link to the server where commands can be send
     */
    public Action(CommunicationChannel com)
    {
	this.com = com;
    }


    /**
     * Returns the first message from the queue (and deletes it)
     *
     * @return oldest message in the queue
     */
    public String getSendMessage()
    {
	if (sendEmpty) return null;
	String entry = sendQueue[sendStart];
	sendQueue[sendStart] = null;
	sendStart = (sendStart+1) % 3;
	if (sendStart==sendEnd) sendEmpty = true;

	return entry;
    }

    /**
     * Writes a new message into the queue
     */
    public void setSendMessage(String aMessage) 
	{
	    sendQueue[sendEnd] = aMessage;
	    sendEnd = (sendEnd + 1) % 3;
	    sendEmpty = false;
	    if ( sendEnd == sendStart ) sendStart = (sendStart + 1) % 3;
	}


    /**
     * Places the player on a given position of the field
     * this is only possible before kick_off and after a goal
     *
     * @param pos is a cartesian vector
     */
     public void move( CVector pos )
    {
	setSendMessage("(move " + pos.x + " " + pos.y + " )");
	send();
    }

    public void move( double x, double y )
    {
	setSendMessage("(move " + x + " " + y + " )");
	send();
    }

    /**
     * The complete player can turn in a given angle
     * this can be positive or negative for different directions
     *
     * @param moment is the angle
     */
    public void turn( int moment )
    {
	setSendMessage("(turn " +moment+ ")");
	send();
    }

    /**
     * The player can accelerate with a given power
     *
     * @param power gives the power of the impulse
     */
    public void dash( double power )
    {
	setSendMessage("(dash " +power+ " )");
	send();
    }

    /**
     * The player can kick in a given direction with a given power
     *
     * @param PVector kickhabit gives the power of the kick
     */
    public void kick( PVector kickhabit )
    {	
	setSendMessage("(kick " +kickhabit.distance+ " " +kickhabit.direction+ ")");
	send();
    }

    /**
     * The player can kick in a given direction with a given power
     *
     * @param power gives the power of the kick
     * @param direction gives the direction of the kick
     */
    public void kick( double power, int direction )
    {	
	setSendMessage("(kick " +power+ " " +direction+ ")");
	send();
    }

    /**
     * The player can turn only his head for looking in another direction
     * 
     * @param angle gives the angle the player turns his head
     */
    public void turn_neck( int angle )
    {
	com.send("(turn_neck " +angle+ " )");
    }

    /**
     * The player can send a message to the other players by saying something
     * the size is the say_msg_size (512)
     * 
     * @param message is a String with a value of 512
     */
     public void say( String message )
    {
	com.send("(say " +message+ " )");
    }

    /**
     * Only the keeper can catch the ball in the penalty area
     * 
     * @param direction gives the direction the keeper should catch
     */
   public void caught( int direction )
    {
        setSendMessage("(catch " +direction+ " )");
	send();
    }

    /**
     * Calls send() when sending of messages is allowed again
     **/
    public void updateTime()
    {
	sendAllowed = true;
	send();
    }


    /**
     * Sends a new messages from the message queue if allowed 
     **/
    private void send()
    {

	if (sendAllowed && !sendEmpty){
	    com.send(getSendMessage());
	    sendAllowed = false;

	}
    }
}

