import java.util.*;


/**
 * Strategy class decides what to do next.
 * Methods which are used by all kind of player types.
 * @author Annika Niehage
 * @author Matthias Lang
 * @author Andrea Barth
 * @version $Id: Strategy.java,v 1.3 2002/07/11 23:26:27 Niehage Exp $
 **/
abstract public class Strategy extends Calculation
{

    /**
     * true if last message was a see message and data was recently calulated<BR>
     * false if we run decide the second time
     **/
    protected boolean currentData;
    protected boolean dashAfterTurn;


/**
 * the constructor initalizes a Strategy object
 * 
 **/
    public Strategy( CommunicationChannel com, Worldmodell data )
    {
	super( com, data);
    }

    /**
     * called after init messages from server<BR>
     * sets initial data like player number and standard positions
     **/
    public void init()
    {
	playerNum = data.getPlayerNum();
	calcStandPosition();
	action.move(startPosition);
	ownPosition = new CVector(startPosition.x, startPosition.y);
    }


    /**
     * called when new body_sense message arrives<BR>
     * sets time and calls action which sends message to server if usefull
     **/
    public void setTime(int timeBody)
    { 
	timeCycle = timeBody;
	action.updateTime();
    }


/**
 * called after see message<BR>
 * calls decide(); every second time it is called twice
 **/
    public void act()
    {
	if (currentData) {
	    decide();
	    currentData = false;
	    decide();
	} else {
	    currentData = true;
	    decide();
	}
    }


/**
 * main strategy method<BR>
 * here decision takes place what to do next
 **/
    public void decide()
    {
	int playMode = data.getPlayMode();
	
	//before kick_off
	if (playMode==9 || playMode==10 || playMode==14) {
	    if (ownPosition.sub(startPosition).length() > 0.5)	    
		action.move(startPosition);
	    else
		turnToBall();
	    return;
	}

	// we don't see the ball
	if (!seeingBall) {
	    if (currentData) turn();
	    return ;
	}

	// we don't know, where we are
	if (!data.seeSomething) {
	    if (dashAfterTurn) {
		action.dash(100);
		ownPosition = ownPosition.add((new PVector( 1, 0)).toCVector(absAngle));
	    }
	    else {
		action.turn(180);
		absAngle = calcAngleModulo(absAngle + 180);
		bodyAngle = calcAngleModulo(bodyAngle + 180);
		dashAfterTurn = true;
	    }
	    return;
	}

	// if we see the ball but may not go to it
	if (playMode > 2 && playMode%2 == 0 && decisionRunToBall() ) {
	    calcDashPowerAndTurn( (new PVector(ballPosition.distance-9.3, ballPosition.direction)).toCVector(absAngle).add(ownPosition) );
	    return;
	}

	// if we see the ball but may not go to it
	if (playMode == 2 && decisionRunToBall()) {
	    calcDashPowerAndTurn( new CVector( 36., absBallPosition.y));
	    CVector absOldBallPosition = new CVector( 36.5, absBallPosition.y/Math.abs(absBallPosition.y)*20);
	    oldBallPosition = new PVector(absOldBallPosition.sub(ownPosition).length(), calcAngleToPos(absOldBallPosition));	    
	    return;
	}

	// we have the ball
	if (ballPosition.distance < KICKABLEDISTANCE) {
	    calcKick();
	    return;
	}

	// shall we run to ball?
	if ( decisionRunToBall() ) {
	    // if (!dashAfterTurn)
	    calcDashPowerAndTurn( ballMeetingPoint );
	    return ;
	}

	adjustPosition();
    }

	




    /**
     * the player decides, if he will run to the ball
     **/
    protected abstract boolean decisionRunToBall();

    /**
     * adjusts position in assigned area
     **/
    protected abstract void adjustPosition();

    /**
     * turns the player to ball
     **/
    protected void turnToBall()
    {
	if(seeingBall)
	{
	    action.turn(ballPosition.direction);
	    absAngle = calcAngleModulo(absAngle + ballPosition.direction);
	    bodyAngle = calcAngleModulo(bodyAngle + ballPosition.direction);
	}
	else
    	    turn();
    }



    /**
     * decides where to turn first if not seeing ball und turns to it
     **/
    protected void turn()
    {
	if(oldBallPosition.direction > 0)
	{
	    action.turn(20);
	    absAngle = calcAngleModulo(absAngle + 20);
	    bodyAngle = calcAngleModulo(bodyAngle + 20);
	
	}

	else
	{
	    action.turn(-20);	 
	    absAngle = calcAngleModulo(absAngle - 20);
	    bodyAngle = calcAngleModulo(bodyAngle - 20);
	}
    }

} // class







